<?php
/*
Plugin Name:WP Widget Cache
Plugin URI: http://www.mashget.com/2008/09/01/wp-widget-cache-for-wordpress/
Description: Cache the output of your blog widgets. Usually it will significantly reduce the sql queries to your database and speed up your site.
Author: Andrew Zhang
Version: 0.15.1
Author URI: http://www.mashget.com
*/

require_once(dirname(__FILE__) . "/inc/wcache.class.php");

class WidgetCache
{
	var $plugin_name="WP Widget Cache";
	var $plugin_version="0.15.1";
	var $plugin_uri="http://www.mashget.com";

	var $wcache;
	var $cachedir;
	var $wgcOptions;
	var $wgcTriggers;

	var $triggerActions=array(
		"category" => array("create_category","edit_category","delete_category"),
		"comment" => array("comment_post","edit_comment","delete_comment","pingback_post"),
		"link" => array("add_link","edit_link","delete_link"),
		"post" => array("delete_post","save_post"),
		"tag" => array("create_term","edit_term", "delete_term")
	);

	function WidgetCache()
	{
		$this->cachedir=dirname(__FILE__)."/cache/";
		$this->wcache = new WCache($this->cachedir);

		//add_action('activate_widget-cache',array(&$this,'widget_cache_warning'));

		if(!(is_dir($this->cachedir)&&is_writable($this->cachedir)))
		{
			add_action('admin_notices', array(&$this,'widget_cache_warning'));
		}
		else
		{
			$this->wgcOptions = get_option('widget_cache');
			if(!$this->wgcOptions) $this->wgcOptions=array();
			//var_dump($this->wgcOptions);

			$this->wgcTriggers=get_option('widget_cache_action_trigger');
			if(!$this->wgcTriggers) $this->wgcTriggers=array();

			//var_dump($this->wgcTriggers);

			add_action('wp_head', array(&$this,'widget_cache_redirect_callback'),1000);

			if(is_admin())
			{
				add_action('sidebar_admin_page', array(&$this,'widget_cache_options_filter'));
				add_action('sidebar_admin_setup', array(&$this, 'widget_cache_expand_control'));
			}
		}
	}

	function widget_cache_warning()
	{
		$wmsg=is_dir($this->cachedir)? "'$this->cachedir' is not writable ":"'$this->cachedir' cann't be created";
		echo "<div id='widget-cache-warning' class='updated fade'><p><strong>WP Widget Cache not work!</strong><br/>$wmsg</p></div>";
	}

	function widget_cache_options_filter()
	{
		$wl_options = $this->wgcOptions;
		?><div class="wrap">
			<form method="POST">
				<a name="wgcoptions"></a>
				<h2><?php  echo $this->plugin_name; ?> Options</h2>
				<p style="line-height: 30px;">
				<span class="submit">					
					<input type="submit" name="widget_cache-clear" id="widget_cache-options-submit" value="Clear all widgets cache(<?php echo $this->wcache->cachecount(); ?>)" />
				</span>
				</p>
			</form>
		</div>
		<?php
	}

	function widget_cache_expand_control()
	{
		global $wp_registered_widgets, $wp_registered_widget_controls;

		$wc_options = $this->wgcOptions;
		$wc_trigers = $this->wgcTriggers;

		if ( 'post' == strtolower($_SERVER['REQUEST_METHOD']) )
		{
			if(isset($_POST["widget_cache-clear"]))
			{
				$this->wcache->clear();
				wp_redirect(add_query_arg( 'message','wgc#wgcoptions'));
				exit();
			}	
				
			//$wc_trigers=array();

			foreach ( (array) $_POST['widget-id'] as $widget_number => $widget_id )
			{
				if (isset($_POST[$widget_id.'-widget_cache']))
				{
					$wc_options[$widget_id]= intval($_POST[$widget_id.'-widget_cache']);
				} 

				if(isset($_POST[$widget_id.'-widget_cache-trigger']))
				{
					$wc_trigers[$widget_id]=($_POST[$widget_id.'-widget_cache-trigger']);
				}
				else 
				{
					unset($wc_trigers[$widget_id]);
				}
			}

			$regd_plus_new=array_merge(array_keys($wp_registered_widgets),array_values((array) $_POST['widget-id']));
			foreach (array_keys($wc_options) as $key)
			{
				if (!in_array($key, $regd_plus_new))
				{
					unset($wc_options[$key]);
					unset($wc_trigers[$key]);
				}
			}

			update_option('widget_cache', $wc_options);
			update_option('widget_cache_action_trigger', $wc_trigers);
			//$this->wcache->clear();
		}
		else
		{
			foreach ( $wp_registered_widgets as $id => $widget )
			{
				if (!$wp_registered_widget_controls[$id])
				wp_register_widget_control($id,$widget['name'], array(&$this, 'widget_cache_empty_control'));

				if (!array_key_exists(0,$wp_registered_widget_controls[$id]['params'])  || is_array($wp_registered_widget_controls[$id]['params'][0]))
				$wp_registered_widget_controls[$id]['params'][0]['id_for_wc']=$id;
				else
				{
					array_push($wp_registered_widget_controls[$id]['params'],$id);
					$wp_registered_widget_controls[$id]['height']+=40;
				}

				$wp_registered_widget_controls[$id]['callback_wc_redirect']=$wp_registered_widget_controls[$id]['callback'];
				$wp_registered_widget_controls[$id]['callback']=array(&$this, 'widget_cache_extra_control');
			}
		}
	}

	function widget_cache_empty_control()
	{

	}

	function widget_cache_extra_control()
	{
		global $wp_registered_widget_controls;
		$params=func_get_args();

		$id=(is_array($params[0]))?$params[0]['id_for_wc']:array_pop($params);
		$id_disp=$id;

		$wc_options = $this->wgcOptions;

		$value=htmlspecialchars(stripslashes($wc_options[$id]),ENT_QUOTES);

		if (is_array($params[0]) && isset($params[0]['number'])) $number=$params[0]['number'];
		if ($number==-1) {$number="%i%"; $value="";}
		if (isset($number)) $id_disp=$wp_registered_widget_controls[$id]['id_base'].'-'.$number;

		$value=intval($value);
		if(!($value>0))$value="";
		?>	
		<fieldset style="border:1px solid #2583AD; padding:3px 0 3px 5px; margin-bottom:10px; line-height:150%;">
			<legend><?php echo $this->plugin_name; ?></legend>
			<div>		
				Expire in <input type='text' 
				name='<?php echo $id_disp; ?>-widget_cache' 
				id='<?php echo $id_disp; ?>-widget_cache' value='<?php echo $value; ?>' size=6 style="padding:0" /> 
				second(s)
				<br/>(Left empty means no cache)
			</div>
			<div style="margin-top:5px; border-top:1px solid #ccc;">
				Auto expire when these things changed:
				<?php
				foreach ($this->triggerActions as $tkey=>$actArr)
				{
					$checked="";
					if($this->wgcTriggers[$id_disp] && in_array($tkey, $this->wgcTriggers[$id_disp]))
					{
						$checked="checked=\"checked\"";
					}
				?>
				<div style="float:left; display:inline; margin:2px 1px 1px 0;" nowrap="nowrap">
					<input type="checkbox" <?php echo $checked;?>  id="<?php echo $id_disp; ?>-widget_cache-trigger-<?php echo $tkey;?>" name="<?php echo $id_disp; ?>-widget_cache-trigger[]" value="<?php echo $tkey;?>" />
					<label for="<?php echo $id_disp; ?>-widget_cache-trigger-<?php echo $tkey;?>"><?php echo ucfirst($tkey);?></label>
				</div>
				<?php
				}
				?>			
			</div>		
		</fieldset>		
		<?php
		$callback=$wp_registered_widget_controls[$id]['callback_wc_redirect'];
		if (is_callable($callback))
		{
			call_user_func_array($callback, $params);
		}
	}

	function widget_cache_redirect_callback()
	{
		global $wp_registered_widgets;
		foreach ( $wp_registered_widgets as $id => $widget )
		{
			array_push($wp_registered_widgets[$id]['params'],$id);
			$wp_registered_widgets[$id]['callback_wc_redirect']=$wp_registered_widgets[$id]['callback'];
			$wp_registered_widgets[$id]['callback']=array(&$this, 'widget_cache_redirected_callback');
		}
	}

	function  widget_cache_getcache($id, $cachetime)
	{
		echo "<!-- WP Widget Cached $id For $cachetime second(s)-->";
		return false;
	}

	function widget_cache_redirected_callback()
	{
		global $wp_registered_widgets;
		$params=func_get_args();											// get all the passed params
		$id=array_pop($params);												// take off the widget ID
		$callback=$wp_registered_widgets[$id]['callback_wc_redirect'];		// find the real callback

		if(!is_callable($callback))return;

		$wc_options = $this->wgcOptions;
		$wc_value=intval($wc_options[$id]);

		if ($wc_value >0)
		{
			echo "<!--$this->plugin_name $this->plugin_version ($this->plugin_uri) Begin -->";
			while($this->wcache->save("wgcache_".$id, $wc_value) || $this->widget_cache_getcache($id, $wc_value))
			{
				call_user_func_array($callback, $params);
			}
			echo "<!--$this->plugin_name End -->";
		}
		else
		{
			call_user_func_array($callback, $params);
		}
	}
}

$widget_cache = & new WidgetCache();

function widget_cache_remove($id)
{
	global $widget_cache;
	$widget_cache->wcache->remove("wgcache_".$id);
}

function widget_cache_hook_trigger()
{
	global $widget_cache;
	if(sizeof($widget_cache->wgcTriggers))
	{
		foreach ($widget_cache->wgcTriggers as $wgid=>$wgacts)
		{
			foreach ($wgacts as $wact)
			{
				if($widget_cache->triggerActions[$wact])
				{
					foreach ($widget_cache->triggerActions[$wact] as $wpaction)
					{
						add_action($wpaction, create_function('$actionarg','widget_cache_remove("'.addslashes($wgid).'");'), 10, 1);
					}
				}
			}
		}
	}
}

widget_cache_hook_trigger();
?>